@@ -1,6 +1,7 @@
 # from __future__ import annotations
 
 from dataclasses import field
+from email import charset
 from typing import Any, Dict, List, Optional, Set
 
 # Use pydantic for type checking. It is slower, but not much of an issue for parsing.
@@ -114,13 +115,18 @@ class PCP():
   rules: List['PCPRule']
   charsets: Dict[str,str] = field(default_factory=DEFAULT_CHARSETS.copy)
   
+  def __post_init__(self):
+    """Validates the PCP after creating it."""
+    self.validate()
+  
   def dumps(self, **kwargs) -> str:
-    """Create a json representation of this object.
+    """Create a json representation of this object. Validates the object before dumping it.
 
     Returns:
         str: Representation of this object in json.
         kwargs: keyword args to pass to json.dumps.
     """
+    self.validate()
     
     # Simplified outputs if features not used
     if self.charsets == DEFAULT_CHARSETS or self.charsets == ALPHABET_CHARSETS:
@@ -183,13 +189,13 @@ class PCP():
     Raises:
         ValueError: Describes how the policy is malformed.
     """
-        
     for key, value in self.charsets.items():
       if len(value) == 0:
         raise ValueError(f'charsets[{key}] must not be empty')
     for charset1, charset2 in itertools.combinations(self.charsets.keys(), 2):
-      if not set(self.charsets[charset1]).isdisjoint(set(self.charsets[charset2])):
-        raise ValueError(f'charsets[{charset1}] and charsets[{charset2}] may not have shared characters')
+      intersection = set(self.charsets[charset1]).intersection(set(self.charsets[charset2]))
+      if len(intersection) > 0:
+        raise ValueError(f'charsets[{charset1}] and charsets[{charset2}] may not have shared characters: {intersection}')
     
     if len(self.rules) < 1:
       raise ValueError(f'rules must contain at least one rule')
@@ -273,21 +279,22 @@ class PCPRule():
       raise ValueError(f'If set, {argname}.max_consecutive may not be less than 1')
     
     if self.require is not None and not self.require.issubset(charsets):
-        raise ValueError(f'{argname}.require includes invalid charsets ({self.require - charsets.keys()})')
+        raise ValueError(f'{argname}.require includes invalid charsets ({self.require - charsets}). Valid charsets are {charsets}.')
     if self.require_subset is not None:
       self.require_subset._validate(f'{argname}.subset_requirement', charsets)
     
     if self.charset_requirements is not None:
       for key, value in self.charset_requirements.items():
         if key not in charsets:
-          raise ValueError(f'{argname}.charset_requirements[{key}] is not a valid charset name')
+          raise ValueError(f'{argname}.charset_requirements[{key}] is not a valid charset. Valid charset are {charsets}.')
         value._validate(f'{argname}.charset_requirements[{key}]', self.min_length)
   
     # Check that require and require_subset don't overlap with each other
     require_charsets = self.require or set()
     subset_charsets = self.require_subset.options or charsets if self.require_subset else set()
-    if len(require_charsets.intersection(subset_charsets)) > 0:
-      raise ValueError(f'In {argname}, require, require_subset, and charset_requirements cannot have overlapping charset requirements ({require_charsets.intersection(subset_charsets)})') 
+    intersection = require_charsets.intersection(subset_charsets)
+    if len(intersection) > 0:
+      raise ValueError(f'In {argname}, require, require_subset, and charset_requirements cannot have overlapping charset requirements ({intersection})') 
     
     # Check that charset_requirements are all valid
     if self.charset_requirements is not None:
@@ -342,14 +349,14 @@ class PCPSubsetRequirement():
       raise ValueError(f'{argname}.count may not be less than 1')
     if self.options is None:
       if self.count >= len(charsets):
-        raise ValueError(f'{argname}.count may not be greater than or equal to the number of available character sets')
+        raise ValueError(f'{argname}.count may not be greater than or equal to the number of available character sets ({len(charsets)})')
     else:
       if len(self.options) < 2:
         raise ValueError(f'{argname}.options must include at least two charsets')
       if not self.options.issubset(charsets):
-        raise ValueError(f'{argname}.options includes invalid charsets ({self.options - charsets.keys()})')
+        raise ValueError(f'{argname}.options includes invalid charsets {self.options - charsets}. Valid charset are {charsets}.')
       if self.count >= len(self.options):
-        raise ValueError(f'{argname}.count may not be greater than or equal to the number of options')
+        raise ValueError(f'{argname}.count may not be greater than or equal to the number of options ({len(self.options)})')
 
 #endregion
 
